@@ -1,6 +1,6 @@
 from abc import abstractproperty
 
-from parso.python.tree import search_ancestor
+from parso.tree import search_ancestor
 
 from jedi import debug
 from jedi import settings
@@ -25,7 +25,7 @@ from jedi.parser_utils import function_is_staticmethod, function_is_classmethod
 
 class InstanceExecutedParamName(ParamName):
     def __init__(self, instance, function_value, tree_name):
-        super(InstanceExecutedParamName, self).__init__(
+        super().__init__(
             function_value, tree_name, arguments=None)
         self._instance = instance
 
@@ -38,7 +38,7 @@ class InstanceExecutedParamName(ParamName):
 
 class AnonymousMethodExecutionFilter(AnonymousFunctionExecutionFilter):
     def __init__(self, instance, *args, **kwargs):
-        super(AnonymousMethodExecutionFilter, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._instance = instance
 
     def _convert_param(self, param, name):
@@ -55,12 +55,12 @@ class AnonymousMethodExecutionFilter(AnonymousFunctionExecutionFilter):
                     self._function_value,
                     name
                 )
-        return super(AnonymousMethodExecutionFilter, self)._convert_param(param, name)
+        return super()._convert_param(param, name)
 
 
 class AnonymousMethodExecutionContext(BaseFunctionExecutionContext):
     def __init__(self, instance, value):
-        super(AnonymousMethodExecutionContext, self).__init__(value)
+        super().__init__(value)
         self.instance = instance
 
     def get_filters(self, until_position=None, origin_scope=None):
@@ -83,15 +83,15 @@ class AnonymousMethodExecutionContext(BaseFunctionExecutionContext):
 
 class MethodExecutionContext(FunctionExecutionContext):
     def __init__(self, instance, *args, **kwargs):
-        super(MethodExecutionContext, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.instance = instance
 
 
 class AbstractInstanceValue(Value):
-    api_type = u'instance'
+    api_type = 'instance'
 
     def __init__(self, inference_state, parent_context, class_value):
-        super(AbstractInstanceValue, self).__init__(inference_state, parent_context)
+        super().__init__(inference_state, parent_context)
         # Generated instances are classes that are just generated by self
         # (No arguments) used.
         self.class_value = class_value
@@ -141,8 +141,7 @@ class CompiledInstance(AbstractInstanceValue):
     # This is not really a compiled class, it's just an instance from a
     # compiled class.
     def __init__(self, inference_state, parent_context, class_value, arguments):
-        super(CompiledInstance, self).__init__(inference_state, parent_context,
-                                               class_value)
+        super().__init__(inference_state, parent_context, class_value)
         self._arguments = arguments
 
     def get_filters(self, origin_scope=None, include_self_names=True):
@@ -234,14 +233,14 @@ class _BaseTreeInstance(AbstractInstanceValue):
         # other way around.
         if is_big_annoying_library(self.parent_context):
             return NO_VALUES
-        names = (self.get_function_slot_names(u'__getattr__')
-                 or self.get_function_slot_names(u'__getattribute__'))
+        names = (self.get_function_slot_names('__getattr__')
+                 or self.get_function_slot_names('__getattribute__'))
         return self.execute_function_slots(names, name)
 
     def py__getitem__(self, index_value_set, contextualized_node):
-        names = self.get_function_slot_names(u'__getitem__')
+        names = self.get_function_slot_names('__getitem__')
         if not names:
-            return super(_BaseTreeInstance, self).py__getitem__(
+            return super().py__getitem__(
                 index_value_set,
                 contextualized_node,
             )
@@ -250,22 +249,17 @@ class _BaseTreeInstance(AbstractInstanceValue):
         return ValueSet.from_sets(name.infer().execute(args) for name in names)
 
     def py__iter__(self, contextualized_node=None):
-        iter_slot_names = self.get_function_slot_names(u'__iter__')
+        iter_slot_names = self.get_function_slot_names('__iter__')
         if not iter_slot_names:
-            return super(_BaseTreeInstance, self).py__iter__(contextualized_node)
+            return super().py__iter__(contextualized_node)
 
         def iterate():
             for generator in self.execute_function_slots(iter_slot_names):
-                for lazy_value in generator.py__next__(contextualized_node):
-                    yield lazy_value
+                yield from generator.py__next__(contextualized_node)
         return iterate()
 
     def py__next__(self, contextualized_node=None):
-        # `__next__` logic.
-        if self.inference_state.environment.version_info.major == 2:
-            name = u'next'
-        else:
-            name = u'__next__'
+        name = u'__next__'
         next_slot_names = self.get_function_slot_names(name)
         if next_slot_names:
             yield LazyKnownValues(
@@ -275,10 +269,10 @@ class _BaseTreeInstance(AbstractInstanceValue):
             debug.warning('Instance has no __next__ function in %s.', self)
 
     def py__call__(self, arguments):
-        names = self.get_function_slot_names(u'__call__')
+        names = self.get_function_slot_names('__call__')
         if not names:
             # Means the Instance is not callable.
-            return super(_BaseTreeInstance, self).py__call__(arguments)
+            return super().py__call__(arguments)
 
         return ValueSet.from_sets(name.infer().execute(arguments) for name in names)
 
@@ -293,10 +287,10 @@ class _BaseTreeInstance(AbstractInstanceValue):
             if result is not NotImplemented:
                 return result
 
-        names = self.get_function_slot_names(u'__get__')
+        names = self.get_function_slot_names('__get__')
         if names:
             if instance is None:
-                instance = compiled.builtin_from_name(self.inference_state, u'None')
+                instance = compiled.builtin_from_name(self.inference_state, 'None')
             return self.execute_function_slots(names, instance, class_value)
         else:
             return ValueSet([self])
@@ -322,7 +316,7 @@ class TreeInstance(_BaseTreeInstance):
             if settings.dynamic_array_additions:
                 arguments = get_dynamic_array_instance(self, arguments)
 
-        super(TreeInstance, self).__init__(inference_state, parent_context, class_value)
+        super().__init__(inference_state, parent_context, class_value)
         self._arguments = arguments
         self.tree_node = class_value.tree_node
 
@@ -396,7 +390,7 @@ class TreeInstance(_BaseTreeInstance):
                 else:
                     if key == index:
                         return lazy_context.infer()
-        return super(TreeInstance, self).py__simple_getitem__(index)
+        return super().py__simple_getitem__(index)
 
     def __repr__(self):
         return "<%s of %s(%s)>" % (self.__class__.__name__, self.class_value,
@@ -407,21 +401,10 @@ class AnonymousInstance(_BaseTreeInstance):
     _arguments = None
 
 
-class CompiledInstanceName(compiled.CompiledName):
-    def __init__(self, inference_state, instance, klass, name):
-        parent_value = klass.parent_context.get_value()
-        assert parent_value is not None, "How? Please reproduce and report"
-        super(CompiledInstanceName, self).__init__(
-            inference_state,
-            parent_value,
-            name.string_name
-        )
-        self._instance = instance
-        self._class_member_name = name
-
+class CompiledInstanceName(NameWrapper):
     @iterator_to_value_set
     def infer(self):
-        for result_value in self._class_member_name.infer():
+        for result_value in self._wrapped_name.infer():
             if result_value.api_type == 'function':
                 yield CompiledBoundMethod(result_value)
             else:
@@ -440,16 +423,12 @@ class CompiledInstanceClassFilter(AbstractFilter):
         return self._convert(self._class_filter.values())
 
     def _convert(self, names):
-        klass = self._class_filter.compiled_value
-        return [
-            CompiledInstanceName(self._instance.inference_state, self._instance, klass, n)
-            for n in names
-        ]
+        return [CompiledInstanceName(n) for n in names]
 
 
 class BoundMethod(FunctionMixin, ValueWrapper):
     def __init__(self, instance, class_context, function):
-        super(BoundMethod, self).__init__(function)
+        super().__init__(function)
         self.instance = instance
         self._class_context = class_context
 
@@ -460,11 +439,11 @@ class BoundMethod(FunctionMixin, ValueWrapper):
     def name(self):
         return FunctionNameInClass(
             self._class_context,
-            super(BoundMethod, self).name
+            super().name
         )
 
     def py__class__(self):
-        c, = values_from_qualified_names(self.inference_state, u'types', u'MethodType')
+        c, = values_from_qualified_names(self.inference_state, 'types', 'MethodType')
         return c
 
     def _get_arguments(self, arguments):
@@ -492,7 +471,7 @@ class BoundMethod(FunctionMixin, ValueWrapper):
         ]
 
     def get_signatures(self):
-        return [sig.bind(self) for sig in super(BoundMethod, self).get_signatures()]
+        return [sig.bind(self) for sig in super().get_signatures()]
 
     def __repr__(self):
         return '<%s: %s>' % (self.__class__.__name__, self._wrapped_value)
@@ -522,17 +501,28 @@ class SelfName(TreeNameDefinition):
     def get_defining_qualified_value(self):
         return self._instance
 
+    def infer(self):
+        stmt = search_ancestor(self.tree_name, 'expr_stmt')
+        if stmt is not None:
+            if stmt.children[1].type == "annassign":
+                from jedi.inference.gradual.annotation import infer_annotation
+                values = infer_annotation(
+                    self.parent_context, stmt.children[1].children[1]
+                ).execute_annotation()
+                if values:
+                    return values
+        return super().infer()
+
 
 class LazyInstanceClassName(NameWrapper):
     def __init__(self, instance, class_member_name):
-        super(LazyInstanceClassName, self).__init__(class_member_name)
+        super().__init__(class_member_name)
         self._instance = instance
 
     @iterator_to_value_set
     def infer(self):
         for result_value in self._wrapped_name.infer():
-            for c in result_value.py__get__(self._instance, self._instance.py__class__()):
-                yield c
+            yield from result_value.py__get__(self._instance, self._instance.py__class__())
 
     def get_signatures(self):
         return self.infer().get_signatures()
@@ -572,7 +562,7 @@ class SelfAttributeFilter(ClassFilter):
     This class basically filters all the use cases where `self.*` was assigned.
     """
     def __init__(self, instance, instance_class, node_context, origin_scope):
-        super(SelfAttributeFilter, self).__init__(
+        super().__init__(
             class_value=instance_class,
             node_context=node_context,
             origin_scope=origin_scope,
@@ -616,10 +606,9 @@ class SelfAttributeFilter(ClassFilter):
 
 class InstanceArguments(TreeArgumentsWrapper):
     def __init__(self, instance, arguments):
-        super(InstanceArguments, self).__init__(arguments)
+        super().__init__(arguments)
         self.instance = instance
 
     def unpack(self, func=None):
         yield None, LazyKnownValue(self.instance)
-        for values in self._wrapped_arguments.unpack(func):
-            yield values
+        yield from self._wrapped_arguments.unpack(func)
